import * as fs from "node:fs";
import path from "node:path";

const docsDir = path.join(process.cwd(), "documentation");

/**
 * Metadata for documentation content.
 */
type DocsContentMetadata = MDXMetadata & {
  /**
   * The title of this content.
   */
  title: string;

  /**
   * The date this content was published.
   */
  published: string;

  /**
   * The summary of this content.
   */
  summary: string;
};

/**
 * Metadata for an MDX file.
 */
type MDXMetadata = {
  /**
   * The slug of the file, defined once read.
   */
  slug: string;

  /**
   * The metadata of the file.
   */
  metadata: {
    [key: string]: string;
  };

  /**
   * The content of the file.
   */
  content: string;
};

/**
 * The regex to match for metadata.
 */
const METADATA_REGEX: RegExp = /---\s*([\s\S]*?)\s*---/;

/**
 * Get the directories in the
 * given directory.
 */
export function getDocsDirectories(dir: string) {
  const dirs: string[] = [dir];
  const paths = fs.readdirSync(dir);

  for (const item of paths) {
    const itemPath = path.join(dir, item);
    const stat = fs.statSync(itemPath);

    if (stat.isDirectory()) {
      dirs.push(...getDocsDirectories(itemPath));
    }
  }
  return dirs;
}

/**
 * Get the content to
 * display in the docs.
 */
export function getDocsContent() {
  const directories = getDocsDirectories(docsDir);
  const content: DocsContentMetadata[] = [];

  for (let directory of directories) {
    content.push(...getMetadata<DocsContentMetadata>(directory));
  }

  return content;
}

export function getDocContent(path?: string[]) {
  const docs = getDocsContent();
  const slug = path ? path.join("/") : "landing";

  return docs.find(doc => doc.slug === slug);
}

/**
 * Get the metadata of mdx
 * files in the given directory.
 *
 * @param directory the directory to search
 */
export function getMetadata<T extends MDXMetadata>(directory: string): T[] {
  const files: string[] = fs.readdirSync(directory).filter((file: string): boolean => {
    const extension: string = path.extname(file); // The file extension
    return extension === ".md" || extension === ".mdx";
  }); // Read the MDX files
  return files.map((file: string): T => {
    const filePath: string = path.join(directory, file); // The path of the file
    return {
      ...parseMetadata<T>(fs.readFileSync(filePath, "utf-8")),
      slug: filePath
        .replace(docsDir, "")
        .replace(/\.mdx?$/, "")
        .replace(/\\/g, "/")
        .substring(1),
    }; // Map each file to its metadata
  });
}

/**
 * Parse the metadata from
 * the given content.
 *
 * @param content the content to parse
 * @returns the metadata and content
 * @template T the type of metadata
 */
function parseMetadata<T extends MDXMetadata>(content: string): T {
  const metadataBlock: string = METADATA_REGEX.exec(content)![1]; // Get the block of metadata
  content = content.replace(METADATA_REGEX, "").trim(); // Remove the metadata block from the content
  let metadata: Partial<{
    [key: string]: string;
  }> = {}; // The metadata to return

  // Parse the metadata block as a key-value pair
  metadataBlock
    .trim() // Trim any leading or trailing whitespace
    .split("\n") // Get each line
    .forEach((line: string): void => {
      const split: string[] = line.split(": "); // Split the metadata by the colon
      let value: string = split[1].trim(); // The value of the metadata
      value = value.replace(/^['"](.*)['"]$/, "$1"); // Remove quotes
      metadata[split[0].trim()] = value; // Add the metadata to the object
    });

  // Return the metadata and content. The initial
  // slug is empty, and is defined later on.
  return { ...metadata, content } as T;
}
